import functools
import os

from google.cloud import secretmanager_v1

from .types import SecretManagerPlugin


class GCPSecretManagerPlugin(SecretManagerPlugin):
    def __init__(self, project_id: str | None = os.getenv("GOOGLE_CLOUD_PROJECT")):
        if not project_id:
            raise ValueError(
                "Google Cloud Project ID must be provided or set in GOOGLE_CLOUD_PROJECT environment variable")
        self.client = secretmanager_v1.SecretManagerServiceClient()
        self.project_id = project_id

    def get_secret(self, secret_name: str) -> str:
        name = f"projects/{self.project_id}/secrets/{secret_name}/versions/latest"
        try:
            response = self.client.access_secret_version(request={"name": name})
            return response.payload.data.decode("UTF-8")
        except Exception as e:
            print(f"Error: {e}")
            raise

    @functools.lru_cache(maxsize=1, typed=True)
    def list_secrets(self, max_results: int = 100) -> list[str]:
        parent = f"projects/{self.project_id}"
        response = self.client.list_secrets(request={"parent": parent, "pageSize": max_results})
        return [secret.name.split('/')[-1] for secret in response]

    def exists(self, secret_name: str, **kwargs) -> bool:
        name = f"projects/{self.project_id}/secrets/{secret_name}"
        try:
            self.client.get_secret(request={"name": name})
            return True
        except Exception as e:
            if "NotFound" in str(e):
                return False
            else:
                raise



import functools

import boto3

from .types import SecretManagerPlugin


class AWSSecretsManagerPlugin(SecretManagerPlugin):
    def __init__(self, region_name: str):
        self.client = boto3.client('secretsmanager', region_name=region_name)

    def get_secret(self, secret_name: str) -> str:
        response = self.client.get_secret_value(SecretId=secret_name)
        return response['SecretString']

    @functools.lru_cache(1, typed=True)
    def list_secrets(self, max_results: int = 100) -> list[str]:
        response = self.client.list_secrets(MaxResults=max_results)
        return [secret['Name'] for secret in response['SecretList']]

    def exists(self, secret_name: str, **kwargs) -> bool:
        return secret_name in self.list_secrets()



import os
import threading
from typing import Optional

from ._secret_manager_default import InMemorySecretManager
from .types import SecretManagerPlugin


class SingletonMeta(type):
    _instances = {}
    _lock = threading.Lock()

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            with cls._lock:
                if cls not in cls._instances:
                    cls._instances[cls] = super(SingletonMeta, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class SecretKeeper(metaclass=SingletonMeta):
    def __init__(self, plugin: SecretManagerPlugin, cache_duration: int = 3600):
        self._manager = InMemorySecretManager(plugin, cache_duration)

    @classmethod
    def from_defaults(cls, cache_duration: int = 3600, vault_url: Optional[str] = None):
        from ._azure_keyvault import AzureKeyVaultPlugin

        vault_url = vault_url or os.environ.get("AZURE_KEY_VAULT_URL")
        if not vault_url:
            raise ValueError("Azure Key Vault URL must be provided or set in AZURE_KEY_VAULT_URL environment variable")

        secret_store = AzureKeyVaultPlugin(vault_url=vault_url)
        return cls(secret_store, cache_duration)

    @classmethod
    def azure(cls, cache_duration: int = 3600, vault_url: Optional[str] = None):
        return cls.from_defaults(cache_duration, vault_url)

    @classmethod
    def aws(cls, cache_duration: int = 3600, region_name: Optional[str] = None):
        from ._aws_secret_manager import AWSSecretsManagerPlugin

        region_name = region_name or os.getenv("AWS_DEFAULT_REGION")
        if not region_name:
            raise ValueError("AWS region must be provided or set in AWS_DEFAULT_REGION environment variable")

        secret_store = AWSSecretsManagerPlugin(region_name=region_name)
        return cls(secret_store, cache_duration)

    @classmethod
    def gcp(cls, cache_duration: int = 3600, project_id: Optional[str] = None):
        from ._gcp_secret_manager import GCPSecretManagerPlugin

        secret_store = GCPSecretManagerPlugin(project_id=project_id) if project_id else GCPSecretManagerPlugin()
        return cls(secret_store, cache_duration)

    def get_secret(self, secret_name: str) -> str:
        secret_value = self._manager.get_secret(secret_name)
        os.environ[secret_name] = secret_value
        return secret_value

    def get(self, secret_name: str) -> str:
        return self.get_secret(secret_name)

    def clear(self, secret_name: str | None = None):
        self._manager.invalidate_cache(secret_name=secret_name)

    # region Popular Gen AI providers
    def get_openai_key(self) -> str:
        return self.get("OPENAI_API_KEY")

    def get_anthropic_key(self) -> str:
        return self.get("ANTHROPIC_API_KEY")

    def get_gemini_key(self) -> str:
        return self.get("GEMINI_API_KEY")

    # endregion


__all__ = ["SecretKeeper", "SecretManagerPlugin"]



"""
A simple version of SecretManager that only caches secrets in process memory.
"""

import logging
import os
import threading
import time
from typing import Any

from .types import SecretManagerPlugin

logger = logging.getLogger(__name__)


class InMemorySecretManager:

    def __init__(self, plugin: SecretManagerPlugin, cache_duration: int = 3600):
        self._cache_lock = threading.Lock()
        self.plugin = plugin
        self.cache_duration = cache_duration
        self.cache: dict[str, dict[str, Any]] = {}

    def get_secret(self, secret_name: str) -> str:
        with self._cache_lock:
            current_time = time.time()

            if secret_name in self.cache:
                cached_secret = self.cache[secret_name]
                if current_time - cached_secret["timestamp"] < self.cache_duration:
                    return cached_secret["value"]

            secret_value = self.plugin.get_secret(secret_name)
            self.cache[secret_name] = {
                "value": secret_value,
                "timestamp": current_time
            }
            return secret_value

    def invalidate_cache(self, secret_name: str | None = None) -> None:
        with self._cache_lock:
            if secret_name is None:
                self.cache.clear()
            elif secret_name in self.cache:
                del self.cache[secret_name]
                if secret_name in os.environ:
                    del os.environ[secret_name]



import abc


class SecretManagerPlugin(abc.ABC):
    @abc.abstractmethod
    def get_secret(self, secret_name: str) -> str:
        pass

    def exists(self, secret_name: str, **kwargs) -> bool:
        raise NotImplementedError

    def list_secrets(self, max_results: int = 100) -> list[str]:
        raise NotImplementedError



"""
Azure Key Vault plugin for SecretKeeper.

This module provides integration with Azure Key Vault for secret management.
It uses Azure's DefaultAzureCredential for authentication with RBAC.

Configuration:
    The following environment variables can be used:
    - AZURE_KEY_VAULT_URL: URL of your Azure Key Vault (required if not passed to constructor)
    - MANAGED_IDENTITY_CLIENT_ID: Client ID for managed identity authentication (optional)

Example:
    >>> from genaikeys import SecretKeeper
    >>> # Using environment variables
    >>> skp = SecretKeeper.azure(vault_url="https://my-vault.vault.azure.net/")
    >>> secret = skp.get("my-secret")
"""

import functools
import os

from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient

from ._secret_manager import SecretManagerPlugin


class AzureKeyVaultPlugin(SecretManagerPlugin):
    def __init__(self, vault_url: str | None = None):
        self.vault_url = vault_url or os.environ.get("AZURE_KEY_VAULT_URL")
        if not self.vault_url:
            raise ValueError("Azure Key Vault URL must be provided or set in AZURE_KEY_VAULT_URL environment variable")
        credential = DefaultAzureCredential(
            managed_identity_client_id=os.environ.get("MANGED_IDENTITY_CLIENT_ID"),
            exclude_interactive_browser_credential=not os.getenv("SECRETKEEPER_DEBUG", "0") == "1"
        )
        # noinspection PyTypeChecker
        self.client = SecretClient(vault_url=self.vault_url, credential=credential)

    @staticmethod
    def _standard_kv_secret_name(secret_name: str) -> str:
        return secret_name.replace("_", "-")

    def get_secret(self, secret_name: str) -> str:
        secret_name = self._standard_kv_secret_name(secret_name)
        secret = self.client.get_secret(secret_name)
        return secret.value

    @functools.lru_cache(maxsize=1, typed=True)
    def list_secrets(self, max_results: int = 100) -> list[str]:
        return [secret.name
                for secret in self.client.list_properties_of_secrets(max_page_size=max_results)]

    def exists(self, secret_name: str, **kwargs) -> bool:
        return secret_name in self.list_secrets(**kwargs)
